/***********************************************************************/
/*                                                                     */
/*                              Io (Java)                              */
/*                                                                     */
/*                  projet Estime, INRIA-Rocquencourt                  */
/*                                                                     */
/*  Copyright 2005-2006 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Inga Berre <Inga.Berre@mi.uib.no>                                  */
/*                                                                     */
/***********************************************************************/

import java.io.*;
import java.io.IOException;
public class CamlCommunication
{
    /**
      We exchange communications as described below (for the time being,
      communications are restricted to one vector of floats or one vector of ints).
   
      Communication::
      begin\n                    (* beginning of communication marker *)
      Items
      end\n                      (* end of communication marker *)
      
      Items:: Vector
   
      Vector::
      [|\n                       (* beginning of vector marker *)
      Type\n                     (* type of the vector items *)
      <Integer>\n                (* size of the vector *)
      Items                      (* items of the vector *)
      |]\n                       (* end of vector marker *)
      
      Type::
      | %i                       (* for integer type *)
      | %f                       (* for float type *)
      
      Items:: Item*
      
      Item:: Lexem;\n

      Lexem::                    (* those are roughly speaking C lexems *)
      | Integer
      | Float

      Integer:: a valid OCaml lexem for integer
      Float:: a valid OCaml lexem for floating point number
    */
    
    public static Object readItem(BufferedReader br) throws IOException{
	boolean readItem = false;	
	String newline = null; 
	String type;
	int size;
	int[] intItem = null;
	double[] doubleItem = null;
	Object item = null; 
	try { 
	    while(readItem == false){
		newline = br.readLine(); 
		if(newline.equals("begin")){
		    readItem = true;
		    newline = br.readLine();
		    if(newline.equals("\"")){ 
			newline = br.readLine();
			size = Integer.parseInt(newline.substring(1,newline.length()-1));
			item = new String();
			for (int i = 0;i<size;i++)
			    item = item + br.readLine().substring(0,1);
		    }
		    else if (newline.equals("[|")){
			newline = br.readLine();
			type = newline.substring(1,2);
			newline = br.readLine();
			size = Integer.parseInt(newline.substring(1,newline.length()-1));
			if(type.equals("i")){
			    //item = new int[size];
			    intItem = new int[size];
			    for (int i = 0;i<size;i++){
				newline= br.readLine();
				intItem[i]=Integer.parseInt(newline.substring(0,newline.length()-1));
			    }
			    item = (Object) intItem;
			}
			else if(type.equals("f")){
			    //item = (double[])item;
			    doubleItem = new double[size];
			    for (int i = 0;i<size;i++){
				newline= br.readLine();
				doubleItem[i]=Double.parseDouble(newline.substring(0,newline.length()-1));
			    }
			    item = (Object) doubleItem;
			}
		    }
		}
	    }
	}
	catch (IOException ioe) {  
	    System.exit(1); 
	} 
	return item;
    }
    public static void writeItem(Object item, BufferedWriter bw) throws IOException{
	
	try{
	    bw.write("begin"); bw.newLine();
	    if (item instanceof int[]){
		int[] intArrayItem = (int[])item;
		bw.write("[|"); bw.newLine();
		bw.write("%i"); bw.newLine();
		bw.write("<"+intArrayItem.length+">"); bw.newLine();
		for(int i=0;i<intArrayItem.length;i++){
		    bw.write(intArrayItem[i]+";"); bw.newLine();
		}
		bw.write("|]\n");
	    }
	    else if (item instanceof double[]){
		double[] doubleArrayItem = (double[])item;
		bw.write("[|"); bw.newLine();
		bw.write("%f"); bw.newLine();
		bw.write("<"+doubleArrayItem.length+">"); bw.newLine();
		for(int i=0;i<doubleArrayItem.length;i++){
		    bw.write(doubleArrayItem[i]+";"); bw.newLine();
		}
		bw.write("|]\n");
	    }
	    else {// if (item instanceof String){
		String stringItem = (String)item;
		bw.write("\""); bw.newLine();
		bw.write("<"+stringItem.length()+">"); bw.newLine();
		for(int i=0;i<stringItem.length();i++){
		    bw.write(stringItem.substring(i,i+1)+";"); bw.newLine();
		}
		bw.write("\"\n");
	    }		
	    bw.write("end"); bw.newLine();
	}
	catch(IOException ioe) {  
	    System.exit(1); 
	}
    }
}
