(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: flags.ml,v 1.3 2011-02-23 17:43:29 fclement Exp $ *)

open Ucommand_option;;

let get_bin, set_bin = Uaref.make_ref true;;

let get_binary () = get_bin ();;
let set_binary () = set_bin true;;
let get_ascii () = not (get_bin ());;
let set_ascii () = set_bin false;;

Ucommand_option.add_bi_all [
  {key = "--ascii_io";
   spec = Arg.Unit set_ascii;
   doc =
     My_command_option.doc_with_default
       "  Turn on the Io library ascii mode: floating-point numbers\
       \n\tare written using an ascii human-readable format.\
       \n\tThis mode is exclusive to binary mode."
       (Printf.sprintf "%b" (get_ascii ()))},
  "-aio";

  {key = "--binary_io";
   spec = Arg.Unit set_binary;
   doc =
     My_command_option.doc_with_default
       "  Turn on the Io library binary mode: floating-point numbers\
       \n\tare written using a compact binary format.\
       \n\tThis mode is exclusive to ascii mode."
       (Printf.sprintf "%b" (get_binary ()))},
  "-bio"
];;
