(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2007-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: sklmldep.ml,v 1.5.2.1 2011-09-21 13:08:03 fclement Exp $ *)

open Install_data;;

let fail msg =
  Printf.eprintf "%s\n" msg; exit 1
;;

let unwords = String.concat " ";;

let mk_command ocamldep_opts pp =
  let pp_opt =
    [ "-pp"; Printf.sprintf "'%s'" (mk_pp_cmd pp); ] in
  unwords ([ "ocamldep"; ] @ pp_opt @ ocamldep_opts)
;;

let parse_argv args =
  let (get_annons, accum_annons) =
      let annons = ref [] in
      (fun () -> !annons),
      (fun px s -> annons := !annons @ [px ^ s])
  and (get_debug, set_debug) =
    let d = ref false in
    (fun () -> !d),
    (fun () -> d := true)
  and (get_pp, set_pp) =
    let pp = ref camlp45_pa_skl in
    (fun () -> !pp),
    (fun s -> pp := s)
  in
  let speclist = Arg.align
    [ "-debug", Arg.Unit set_debug,
      " Toggle display of debug informations during compilation"
    ; "-ext", Arg.String (accum_annons "-"),
      " Internal use only"
    ; "-pp", Arg.String set_pp,
      "PP Sets the location of the camlp4/5 parser module to use"
    ] in
  let keys =
    [ "-help"; "--help"; ] @ List.map (fun (x, _, _) -> x) speclist in
  let args = Array.of_list
    (Array.fold_left (fun l s ->
      if not (List.mem s keys) then
        let key = String.sub s 1 (String.length s - 1) in
        if s.[0] = '-'
        then l @ [ "-ext"; key ]
        else l @ [s]
      else l @ [s]) [] args) in
  try
    Arg.parse_argv args speclist (accum_annons "")
      ("The Sklml dependency generator will accept all" ^
       " ocamldep options plus these ones:\n");
    (get_annons (), get_pp (), get_debug ())
  with
  | Arg.Bad _ -> fail "invalid usage, see help using -help option"
  | Arg.Help m -> print_string m; exit 0
  | Failure m -> fail ("invalid usage, " ^ m)
;;

let dbprintf b =
  (if b then Printf.fprintf else Printf.ifprintf) stdout
;;

let main () =
  let (ocamldep_opts, pp, dbg) = parse_argv Sys.argv in
  let compile_cmd = mk_command ocamldep_opts pp in
  dbprintf dbg "will run: %s\n" compile_cmd;
  let ret = Sys.command compile_cmd in
  (if ret = 0
  then dbprintf dbg "dependencies generated\n"
  else dbprintf dbg "there were errors\n");
  exit ret
;;

let _ = main ();;
